package org.blue.club.services;

import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryCondition;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.blue.club.dao.ClubDao;
import org.blue.club.dao.ClubUserAuthDao;
import org.blue.club.dao.UserDao;
import org.blue.club.entities.dto.club.req.ClubAddUserRequest;
import org.blue.club.entities.dto.club.req.ClubChangeUserAuthRequest;
import org.blue.club.entities.dto.club.req.CreateClubRequest;
import org.blue.club.entities.dto.user.User;
import org.blue.club.entities.vo.data.ClubAuthVo;
import org.blue.club.entities.vo.data.ClubUserAuthVo;
import org.blue.club.entities.vo.data.ClubVo;
import org.blue.club.entities.vo.data.UserVo;
import org.mmga.spring.boot.starter.entities.PagerData;
import org.mmga.spring.boot.starter.entities.Result;
import org.mmga.spring.boot.starter.exception.AuthorizationException;
import org.mmga.spring.boot.starter.utils.VoUtils;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static com.mybatisflex.core.query.QueryMethods.select;
import static org.blue.club.entities.vo.data.table.ClubUserAuthVoTableDef.CLUB_USER_AUTH_VO;
import static org.blue.club.entities.vo.data.table.ClubVoTableDef.CLUB_VO;
import static org.blue.club.entities.vo.data.table.UserVoTableDef.USER_VO;

@Service
@RequiredArgsConstructor
@Slf4j
public class ClubServices {
    private final ClubDao clubDao;
    private final ClubUserAuthDao clubUserAuthDao;
    private final UserDao userDao;
    private final VoUtils voUtils;

    public Result<PagerData<ClubVo>> getClubs(Integer page, Integer num) {
        Page<ClubVo> data = clubDao.paginate(page, num, QueryCondition.createEmpty());
        return Result.success(new PagerData<>(data.getTotalRow(), data.getRecords()));
    }

    public Result<ClubVo> createClub(CreateClubRequest request) {
        if (clubDao.selectCountByCondition(CLUB_VO.NAME.eq(clubDao)) > 0)
            return Result.failed(HttpStatus.CONFLICT, "社团名称已存在！");
        ClubVo clubVo = new ClubVo();
        clubVo.setName(request.name());
        return Result.success(clubVo);
    }

    @Transactional
    public Result<Boolean> clubAddUser(User user, ClubAddUserRequest request) {
        long requestClubId = request.clubId();
        long userClubId = -1;
        ClubVo userClub = user.getClub();
        if (userClub != null) {
            userClubId = userClub.getId();
        }
        if (userClubId != requestClubId && !user.hasPermission(3))
            throw new AuthorizationException(Result.failed(HttpStatus.FORBIDDEN, "权限不足！"));
        long userClubAuthId = -1;
        ClubAuthVo clubAuth = user.getClubAuth();
        if (clubAuth != null) {
            userClubAuthId = clubAuth.getId();
        }
        long requestClubAuthId = request.clubAuthId();
        if (requestClubAuthId >= userClubAuthId && !user.hasPermission(3))
            throw new AuthorizationException(Result.failed(HttpStatus.FORBIDDEN, "权限不足！"));
        long requestUserId = request.userId();
        if (clubUserAuthDao.selectCountByCondition(CLUB_USER_AUTH_VO.USER_ID.eq(requestUserId)) > 0)
            return Result.failed(HttpStatus.CONFLICT, "此用户已在社团中");
        ClubUserAuthVo clubUserAuthVo = new ClubUserAuthVo();
        clubUserAuthVo.setClubId(requestClubId);
        clubUserAuthVo.setUserId(requestUserId);
        clubUserAuthVo.setAuthId(requestClubAuthId);
        clubUserAuthDao.insert(clubUserAuthVo);
        return Result.success(true);
    }

    public Result<PagerData<User>> clubGetUsers(Integer page, Integer num, Long clubId) {
        if (clubDao.selectCountByCondition(CLUB_VO.ID.eq(clubId)) == 0)
            return Result.failed(HttpStatus.NOT_FOUND, "社团不存在！");
        Page<ClubUserAuthVo> paginate = clubUserAuthDao.paginate(page, num, select(CLUB_USER_AUTH_VO.ALL_COLUMNS).where(CLUB_USER_AUTH_VO.CLUB_ID.eq(clubId)).orderBy(CLUB_USER_AUTH_VO.AUTH_ID.asc(), CLUB_USER_AUTH_VO.USER_ID.asc()));
        List<UserVo> userVos = userDao.selectListWithRelationsByQuery(select(USER_VO.ALL_COLUMNS).from(USER_VO).where(USER_VO.ID.in(paginate.getRecords().stream().map(ClubUserAuthVo::getUserId).toList())));
        return Result.success(new PagerData<>(paginate.getTotalRow(), userVos.stream().map(e -> voUtils.vo2DtoSafe(e, User.class)).toList()));
    }

    public Result<Boolean> clubRemoveUser(User user, long clubId, long userId) {
        ClubUserAuthVo clubUserAuthVo = clubUserAuthDao.selectOneByCondition(CLUB_USER_AUTH_VO.USER_ID.eq(userId).and(CLUB_USER_AUTH_VO.CLUB_ID.eq(clubId)));
        if (clubUserAuthVo == null) return Result.failed(HttpStatus.NOT_FOUND, "未知用户");
        Long authId = clubUserAuthVo.getAuthId();
        ClubAuthVo clubAuth = user.getClubAuth();
        long clubAuthId = -1;
        if (clubAuth != null) {
            clubAuthId = clubAuth.getId();
        }
        if (clubAuthId <= authId && !user.hasPermission(3))
            throw new AuthorizationException(Result.failed(HttpStatus.FORBIDDEN, "权限不足！"));
        clubUserAuthDao.delete(clubUserAuthVo);
        return Result.success(true);
    }

    public Result<Boolean> clubChangeUserAuth(User user, ClubChangeUserAuthRequest request) {
        Long userId = request.userId();
        Long clubId = request.clubId();
        ClubUserAuthVo clubUserAuthVo = clubUserAuthDao.selectOneByCondition(CLUB_USER_AUTH_VO.USER_ID.eq(userId).and(CLUB_USER_AUTH_VO.CLUB_ID.eq(clubId)));
        if (clubUserAuthVo == null) return Result.failed(HttpStatus.NOT_FOUND, "未知用户");
        // TODO
        // 不能修改原始权限比操作者更高的用户
        // 当目标权限比操作者更高时不允许修改

        return null;
    }
}
